<?

 /*************************************************************************************
 * Filename: modules/entries.php
 *
 * This file is part of the Blog JAS plugin.
 * Copyright 2003-2015 by Matthijs Schuurman
 *
 * The Blog JAS plugin is free software under the terms of the
 * GNU General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the plugin or http://jas.junerules.com/plugins/blog/
 *************************************************************************************/


 require_once($_path["include"]."module.php");

 class Plugin_Module_blog_entries extends Module
 {
  function __construct(&$data)
  {
   parent::__construct($data);

   $this->data->title="Blog entries";
   $this->data->addField("published", "boolean", "all", "", 0, false, true, "Published");
   $this->data->addField("date", "datetime", "all", "", 0, false, "now", "Date");
   $this->data->addField("subject", "text", "all", "title", 1, 255, false, "Subject");
   $this->data->addField("message", "html", "all", "wysiwyg", 0, 65536, false, "Message");
   $this->data->addField("tags", "text", "show;form", "", 1, 255, false, "Tags");

   $this->data->defaultFields();

   
   $this->data->addReference("comments", "blog_comments", "blogentrynr", "1:*", "nr");

   if(isset($this->data->field["comments"]))
    $this->data->field["comments"]["properties"].=";hidden";


   $this->data->addReference("blog_tagsmapping", "blog_tags", "nr", "*:*", "nr");

   if(isset($this->data->field["blog_tagsmapping"]))
    $this->data->field["blog_tagsmapping"]["properties"].=";hidden";


   $this->data->addFullTextIndex("fulltext", "subject", "message");

   $this->data->sortOrder("date", "subject");
  }

  function viewPostGet(){return $this->convertTagsToText();}
  function copyPostGet(){return $this->convertTagsToText();}
  function editPostGet(){return $this->convertTagsToText();}
  function movePostGet(){return $this->convertTagsToText();}
  function deletePostGet(){return $this->convertTagsToText();}
  function ownerPostGet(){return $this->convertTagsToText();}

  function convertTagsToText()
  {
   if($this->data->check("use", "show", "tags"))
   {
    $d=&$this->data->getReferenceData("blog_tagsmapping"); //get blog_tagsmapping

    //setup blog_tags reference
    $dr=&$d->getReferenceData("blog_tags");
    $showFields=$dr->check("use", "show"); //store show fields
    $dr->setShowFields("tag");

    for($c=0;$c<$this->data->vmax;$c++)
    {
     $d->setAction("view");
     $d->setExtra("all");
     $d->clearValues();
     $d->setValue($this->data->module, $this->data->getValue("nr", $c));

     $tags=Array();
     if($d->get(1))
     {
      while($d->next())
       $tags[]=$d->v["blog_tags"]["tag"];
     }

     $this->data->setValue("tags", implode(", ", $tags), $c);
    }

    $dr->setShowFields($showFields); //restore show fields
   }

   return true;
  }

  function viewPreGetErrorCheck(){return $this->convertTextToTags();}
  function addPreSubmitErrorCheck(){return $this->convertTextToTagsAndAdd();}
  function editPreSubmitErrorCheck(){return $this->convertTextToTagsAndAdd();}

  function convertTextToTags()
  {
   $tagsmapping=0;
   for($c=0;$c<$this->data->vmax;$c++)
   {
    if(($text=$this->data->getValue("tags", $c))&&preg_match_all("/([^,]+),?\s*/", $text, $m)) //match comma separated list of tags
    {
     $text=$m[1];
     $tags=Array();

     if(!isset($d)) //init tags data object
     {
      $d=Data::cache("blog_tags", "view", "all");
      $d->setShowFields("nr");
     }

     $x=0;
     $d->clearValues();
     foreach($text as $tag) //find correspondig tag nr
     {
      $d->setValue("tag", $tag, $x);
      $d->setOperant("tag", "OR", $x);
      $x++;
     }

     if(!$d->get())
     {
      $this->data->e->add($d->e);
      return false;
     }
     else
     {
      if($d->vmax==0) //tag(s) not found
      {
       $this->data->setValue("blog_tagsmapping", 0, $tagsmapping); //ensure no results are return (unless other tags are found)
       $tagsmapping++;
      }
      else
      {
       while($d->next()) 
       {
        if($tagsmapping>0)
         $this->data->setOperant("blog_tagsmapping", "OR", $tagsmapping-1);

        $this->data->setValue("blog_tagsmapping", $d->v["nr"], $tagsmapping);
        $tagsmapping++;
       }
      }
     }
    }
   }

   return true;
  }


  function convertTextToTagsAndAdd()
  {
   for($c=0;$c<$this->data->vmax;$c++)
   {
    if(($text=$this->data->getValue("tags", $c))&&preg_match_all("/([^,]+),?\s*/", $text, $m)) //match comma separated list of tags
    {
     $text=$m[1];
     $tags=Array();

     if(!isset($d)) //init tags data object
     {
      //get catnr for tags in edit mode
      if($this->data->action=="edit"&&$this->data->getValueType("catnr")=="default")
      {
       $d=Data::cache($this->data->module, "view", "item");
       $d->setValue("nr", $this->data->getValue("nr"));
       $d->setShowFields("catnr");

       if($d->get()&&$d->next())
        $this->data->setValue("catnr", $d->v["catnr"]);

       unset($d);
      }

      $d=Data::cache("blog_tags", "view", "all");
      $d->setShowFields("nr");
     }

     foreach($text as $tag) //find correspondig tag nr
     {
      $d->clearValues();
      $d->setValue("tag", $tag);

      if($d->get()&&$d->vmax) //tag exists
       $tags[]=$d->getValue("nr"); //store tag+id
      else
      {
       //add tag to database
       $d->setAction("add");
       $d->setExtra("item");

       $d->clearValues();
       $d->setValue("tag", $tag);
       $d->setValue("catnr", $this->data->getValue("catnr"));

       if(!$d->submit())
       {
        $this->data->e->add($d->e);
        return false;
       }

       $tags[]=$d->getValue("nr"); //store tag+id

       //reset data object
       $d->setAction("view"); 
       $d->setExtra("all");
      }
     }

     $this->data->setValue("blog_tagsmapping", $tags, $c);
    }
   }

   return true;
  }

 }
?>
